/*
	Copyright (c) 2010 by Dennis Schridde

	This file is part of dovecot-metadata.

	dovecot-metadata is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	dovecot-metadata is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with dovecot-metadata.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef DOVECOT_METADATA_SETTINGS_H
#define DOVECOT_METADATA_SETTINGS_H

#include "metadata-global.h"

#include "mail-user.h"

struct metadata_settings {
	bool metadata_debug;
	const char *dict_uri;
	int maxsize;
	int maxentries;
};

void
metadata_settings_init(struct metadata_settings **set, struct mail_user *user)
	ATTR_NONNULL(1,2);
void
metadata_settings_deinit(struct metadata_settings **set)
	ATTR_NONNULL(1);

#endif
