/*
	Copyright (c) 2010 by Dennis Schridde

	This file is part of dovecot-metadata.

	dovecot-metadata is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	dovecot-metadata is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with dovecot-metadata.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef DOVECOT_STR_EXT_H
#define DOVECOT_STR_EXT_H

#include "metadata-global.h"

#include <stdbool.h>

#include "str.h"

#define str_append_printf str_printfa

int
strchr_num(const char *s, int c)
	ATTR_NONNULL(1);

bool
str_has_wildcards(const char *s)
	ATTR_NONNULL(1);

#endif
