/*
	Copyright (c) 2010 by Dennis Schridde

	This file is part of dovecot-metadata.

	dovecot-metadata is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	dovecot-metadata is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with dovecot-metadata.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef DOVECOT_METADATA_MAIL_USER_MODULE_PRIVATE_H
#define DOVECOT_METADATA_MAIL_USER_MODULE_PRIVATE_H

#include "metadata-mail-user-module.h"

#include "dict.h"
#include "module-context.h"
#include "mail-user.h"

#include "metadata-settings.h"

#define METADATA_USER_CONTEXT(obj) MODULE_CONTEXT(obj, metadata_mail_user_module)

extern MODULE_CONTEXT_DEFINE(metadata_mail_user_module, &mail_user_module_register);

struct metadata_mail_user {
	union mail_user_module_context module_ctx;
	struct dict *dict;
	struct metadata_settings *set;
};

#endif
